/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef BAIDUSPEECHENGINE_P_H
#define BAIDUSPEECHENGINE_P_H

#include <kylin-ai/ai-engine/large-model/cloudspeechengine.h>
#include <thread>
#include <cpr/cpr.h>
#include <curl/curl.h>

class BaiduSpeechEnginePrivate : public ai_engine::lm::speech::AbstractCloudSpeechEngine
{
public:
    BaiduSpeechEnginePrivate();
    ~BaiduSpeechEnginePrivate();
    std::string backendName() override;

    void setAppId(const std::string &appId) override;
    void setApiKey(const std::string &apiKey) override;
    void setSecretKey(const std::string &secretKey) override;

    void setRecognizingCallback(ai_engine::lm::speech::RecognitionResultCallback callback) override;
    void setRecognizedCallback(ai_engine::lm::speech::RecognitionResultCallback callback) override;

    void setSynthesizingCallback(ai_engine::lm::speech::SynthesisResultCallback callback) override;
    void setSynthesizedCallback(ai_engine::lm::speech::FinishedCallback callback) override;

    // 语音识别
    ai_engine::ErrorCode initContinuousRecognitionModule() override;
    ai_engine::ErrorCode startContinuousRecognition(const std::string &params) override;
    ai_engine::ErrorCode writeContinuousRecognitionAudioData(const std::vector<char> &audioData) override;
    ai_engine::ErrorCode stopContinuousRecognition() override;
    ai_engine::ErrorCode destroyContinuousRecognitionModule() override;

    ai_engine::ErrorCode initRecognizeOnceModule() override;
    ai_engine::ErrorCode recognizeOnce(const std::string &params, const std::vector<char> &audioData) override;
    ai_engine::ErrorCode destroyRecognizeOnceModule() override;

    // 语音合成
    ai_engine::ErrorCode initContinuousSynthesisModule() override;
    ai_engine::ErrorCode startContinuousSynthesis(const std::string &params) override;
    ai_engine::ErrorCode writeContinuousSynthesisText(const std::string &text) override;
    ai_engine::ErrorCode stopContinuousSynthesis() override;
    ai_engine::ErrorCode destroyContinuousSynthesisModule() override;

    ai_engine::ErrorCode initSynthesizeOnceModule() override;
    ai_engine::ErrorCode synthesizeOnce(const std::string &params, const std::string &text) override;
    ai_engine::ErrorCode destroySynthesizeOnceModule() override;

private:
    std::string appId_;
    std::string apiKey_;
    std::string secretKey_;
    std::string accessToken_;
    bool needRefreshAccessToken_{false};
    std::string &getAccessToken_();
    void generateAccessToken_();

    ai_engine::lm::speech::RecognitionResultCallback recognizingCallback_;
    ai_engine::lm::speech::RecognitionResultCallback recognizedCallback_;
    ai_engine::lm::speech::SynthesisResultCallback synthesizingCallback_;
    ai_engine::lm::speech::FinishedCallback synthesizedCallback_;

    std::string format_;
    int rate_{0};
    int channel_{0};
    void setRecognitionParams_(const std::string &params);
    friend bool writeOnceRecognitionResult(std::string data, intptr_t userData);
    friend bool doSendContinuousRecognitionSourceData(BaiduSpeechEnginePrivate *engine,
                                                      int type, const char *data,
                                                      int dataLength);
    friend bool doReceiveContinuousRecognitionResultData(BaiduSpeechEnginePrivate *);
    friend bool connectWebSocket(CURL *curl, BaiduSpeechEnginePrivate *);
    friend void onWebSocket(bool (*doReceiveData)(BaiduSpeechEnginePrivate *),
                            BaiduSpeechEnginePrivate *);
    friend void closeWebsocket(CURL *curl);

    int speed_{0};
    int volume_{0};
    int pitch_{0};
    void setSynthesisParams_(const std::string &params);
    ai_engine::ErrorCode doRequestSynthesize(const std::string &text, std::string type);
    friend bool writeSynthesisResult(std::string data, intptr_t userData);

    bool initWebResources();
    void releaseWebResources();
    bool processCprResponse_(cpr::Response &response, std::string type);

    bool isStopped_{false};

    CURL *currentCurl_{nullptr};
    std::thread *currentThread_{nullptr};

    int resultCode_{0};
    std::string lastErrorMessage_;
};

#endif
